import os

import ftrack_api
from pick import pick


server = "https://ams.ftrackapp.com/api?"


def create_project_name(num, client, name):
    return f"{num}_{client.upper()}_{name.upper()}"


def get_yes_no(question):
    answer = pick(["Yes", "No"], question)
    return True if answer[1] == 0 else False


def get_int_from_user(question):
    while True:
        try:
            intval = int(input(question))
        except ValueError:
            print("Sorry, I didn't understand that? Did you type a number?")
            continue
        else:
            return intval


def clearConsole():
    return os.system("cls" if os.name in ("nt", "dos") else "clear")


def get_ftrack_session():
    return ftrack_api.Session(
        server_url=server,
        api_key=os.getenv("FTRACK_API_KEY"),
        api_user=os.getenv("FTRACK_API_USER"),
    )


def select_artist(team, users, question):
    ia = pick([u["username"] for u in team if u["username"] not in [
              "Hanne", "Nele", "Annelies", "Pieter"]], question, indicator="👉")
    return [i for i in users if ia[0].lower() in i["username"].lower()][0]


def number_to_letter(input: int) -> str:
    return str(chr(ord("@") + input + 1))


def shotnumber_to_letter(input: int) -> str:
    res = divmod(input, 26)
    quotient = res[0]
    remainder = res[1]
    if quotient > 0:
        return f"{number_to_letter(quotient-1)}{number_to_letter(remainder)}"
    else:
        return number_to_letter(input)



import click
from pick import pick

from ftrack_ams.fileserver import create_project_on_fileserver, get_deal_info, get_latest_proj
from ftrack_ams.functions import clearConsole, get_int_from_user, select_artist
from ftrack_ams.functions import create_project_name, get_yes_no


def create_new_project(session):

    continue_project_creation = get_yes_no(
        "Are you sure you want to create a new project? 🕵️‍♂️"
    )

    if not continue_project_creation:
        print("Ok bye 👋")
        quit()

    projects = session.query("Project")
    users = session.query("User")
    client = None
    projname = None

    logo_handle = session.query('Folder where name is "AMSVFB_Logo"').one()

    for logo in logo_handle["children"]:
        if logo["name"] == "VFB-LOGO":
            vfb_thumb = logo["thumbnail_id"]
        if logo["name"] == "AMSVFB-LOGO":
            amsvfb_thumb = logo["thumbnail_id"]
        if logo["name"] == "AMS-LOGO":
            ams_thumb = logo["thumbnail_id"]

    clearConsole()
    num = None
    dealnumber = None
    deal = None
    if get_yes_no("Do you have a deal number?"):
        while True:
            dealnumber = input("👀 Enter deal number: ")
            try:
                int(dealnumber)
            except ValueError:
                print("🤯 Sorry, I didn't understand that? Did you type a number?")
                continue
            else:
                if len(dealnumber) == 4:
                    click.secho(
                        f"Looks like {dealnumber} already existing, try adding to this project instead of creating a new one", fg="yellow")
                    input("press enter to re-run")
                    quit()
                if len(dealnumber) == 3:
                    click.secho("nieuw project op basis van een deal, cool story brah")
                    break
                else:
                    click.secho("🤯 Try typing 3 or 4 numbers", fg="red")
                    continue

    if dealnumber is not None:
        deal = get_deal_info(dealnumber)
        if deal is not None:
            split = deal.split('_')
            client = split[3]
            projname = split[4]

    print(f"👍 Ok {session.api_user}, which team are we making a project for?")

    option, index = pick(
        sorted([p["name"] for p in projects if "invoice" not in p["name"]]),
        "👬 Select the team:",
        indicator="👉",
    )

    clearConsole()

    num = get_latest_proj()
    if get_yes_no(f"🤹‍♂️ Based on the latest X: project, your project number is {num}, u like?"):
        print(f"🤩 Ok, we're rolling with {num}")
    else:
        while True:
            raw_input = input("😮 Ok, enter desired project number: ")
            try:
                int(raw_input)
            except ValueError:
                print("Sorry, I didn't understand that? Did you type a number?")
                continue
            else:
                if len(raw_input) != 4:
                    click.secho("🤯 Try typing 4 numbers", fg="red")
                    continue
                num = raw_input
                break

    team = [t for t in projects if option.lower() == t["name"].lower()][0]
    print(f"💁‍♂️ Making a new project for {team['name']}")

    mgmt = []
    for m in team["managers"]:
        if "Annelies" not in m["user"]["username"]:
            mgmt.append(m["user"])
        else:
            annelies = m["user"]

    project_team = []

    for a in team["allocations"]:
        resource = a["resource"]
        if isinstance(resource, session.types["User"]):
            user = resource
            project_team.append(user)

    internal_project = True if "Intern" in team["name"] else False

    if internal_project:
        departement_choice, departement_index = pick(
            ["AMS", "VFB"], "Select internal department:"
        )

    project_thumb = amsvfb_thumb

    if internal_project and departement_choice == "AMS":
        project_thumb = ams_thumb

    if internal_project and departement_choice == "VFB":
        project_thumb = vfb_thumb

    dest_folder = None
    for child in team["children"]:
        if child["name"] == "Projects":
            dest_folder = child

    if dest_folder is None:
        click.secho(
            f"could not find a 'Projects' folder for team {team['name']}", fg="red")
        quit()
    if client is None:
        while True:
            client = input("Enter client letter code (3 characters): ")
            if len(client) == 3:
                print(f"✨ Making project {num} for client {client}")
                break
            else:
                print("didnt get ya fully")
                continue
    if projname is None:
        while True:
            projname = input("Enter project letter code (3 characters):")
            if len(projname) == 3:
                break
            else:
                print("Didnt get ya fully? Type three characters.")
                continue

    project_name = create_project_name(num, client, projname)

    num_int = get_int_from_user("Enter amount of INT shots: ")
    num_ext = get_int_from_user("Enter amount of EXT shots: ")
    num_moving = get_int_from_user("Enter amount of MOVING IMAGES: ")

    if num_int == 0:
        desc = f"{num_ext} EXT"
    elif num_ext == 0:
        desc = f"{num_int} INT"
    else:
        desc = f"{num_int} INT/{num_ext} EXT"

    if internal_project and departement_choice == "VFB":
        proj = session.create(
            "Vfbproj",
            {
                "name": project_name,
                "parent": dest_folder,
                "description": desc,
                "thumbnail_id": project_thumb,
            },
        )
    else:
        proj = session.create(
            "Amsproj",
            {
                "name": project_name,
                "parent": dest_folder,
                "description": desc,
                "thumbnail_id": project_thumb,
            },
        )

    task_templates = team["project_schema"]["task_templates"]

    for template in task_templates:
        if template["name"] == "Annelies_Template":
            annelies_template = template
        if template["name"] == "Image_Template":
            image_template = template
        if template["name"] == "PM_Template":
            production_template = template
        if template["name"] == "Timetracking_Template":
            timetrack_template = template
        if template["name"] == "MovingImage_Template":
            moving_image_template = template

    pm_choice, index = pick([m["username"] for m in mgmt], "👩‍🏫 Select project manager")
    project_manager = mgmt[index]

    for task_type in [t["task_type"] for t in production_template["items"]]:
        task = session.create(
            "Task",
            {
                "name": task_type["name"],
                "type": task_type,
                "thumbnail_id": project_thumb,
                "parent": proj,
            },
        )
        session.create(
            "Appointment",
            {"context": task, "resource": project_manager, "type": "assignment"},
        )

    for task_type in [t["task_type"] for t in annelies_template["items"]]:
        task = session.create(
            "Task",
            {
                "name": task_type["name"],
                "type": task_type,
                "thumbnail_id": project_thumb,
                "parent": proj,
            },
        )

        session.create(
            "Appointment", {"context": task, "resource": annelies, "type": "assignment"}
        )

    if num_int > 0:
        int_folder = session.create(
            "Folder",
            {"name": f"{num}_INT",
             "parent": proj,
             "thumbnail_id": project_thumb}
        )

        if len(project_team) > 1:
            int_artist = select_artist(project_team, users, "👩‍🎨🛋 Select INT artist: ")
        else:
            int_artist = project_team[0]

        for i in range(num_int):
            int_shot_name = f"{num}_INT_{chr(ord('@')+i+1)}"

            int_shot = session.create(
                "Image",
                {
                    "name": int_shot_name,
                    "parent": int_folder,
                    "thumbnail_id": project_thumb,
                },
            )

            for task_type in [t["task_type"] for t in image_template["items"]]:
                task = session.create(
                    "Task",
                    {
                        "name": task_type["name"],
                        "type": task_type,
                        "parent": int_shot,
                        "thumbnail_id": project_thumb,
                    },
                )
                session.create(
                    "Appointment",
                    {"context": task, "resource": int_artist, "type": "assignment"},
                )

    if num_ext > 0:
        ext_folder = session.create(
            "Folder",
            {"name": f"{num}_EXT", "parent": proj, "thumbnail_id": project_thumb},
        )

        if len(project_team) > 1:
            ext_artist = select_artist(project_team, users, "👨‍🎨🏞 Select EXT artist: ")
        else:
            ext_artist = project_team[0]

        for i in range(num_ext):
            ext_shot_name = f"{num}_EXT_{chr(ord('@')+i+1)}"
            ext_shot = session.create(
                "Image",
                {
                    "name": ext_shot_name,
                    "parent": ext_folder,
                    "thumbnail_id": project_thumb,
                },
            )
            for task_type in [t["task_type"] for t in image_template["items"]]:
                task = session.create(
                    "Task",
                    {
                        "name": task_type["name"],
                        "type": task_type,
                        "parent": ext_shot,
                        "thumbnail_id": project_thumb,
                    },
                )
                session.create(
                    "Appointment",
                    {"context": task, "resource": ext_artist, "type": "assignment"},
                )

    if num_moving > 0:
        moving_image_folder = session.create(
            "Folder",
            {"name": f"{num}_MovingImage", "parent": proj,
             "thumbnail_id": project_thumb},
        )

        if len(project_team) > 1:
            movingImage_artist = select_artist(
                project_team, users, "👨‍🎨🏞 Select MovingImage artist: ")
        else:
            movingImage_artist = project_team[0]

        for i in range(num_moving):
            moving_image_name = f"{num}_MovingImage_{chr(ord('@')+i+1)}"
            moving_image_shot = session.create(
                "Movingimage",
                {
                    "name": moving_image_name,
                    "parent": moving_image_folder,
                    "thumbnail_id": project_thumb,
                },
            )
            for task_type in [t["task_type"] for t in moving_image_template["items"]]:
                task = session.create(
                    "Task",
                    {
                        "name": task_type["name"],
                        "type": task_type,
                        "parent": moving_image_shot,
                        "thumbnail_id": project_thumb,
                    },
                )
                session.create(
                    "Appointment",
                    {"context": task, "resource": movingImage_artist, "type": "assignment"},
                )

    # using a set here to make sure users aren't added multiple times
    project_set = set()
    if num_int > 0:
        project_set.add(int_artist)
    if num_ext > 0:
        project_set.add(ext_artist)
    if num_moving > 0:
        project_set.add(movingImage_artist)

    if internal_project:
        print("Internal project, creating timetracking tasks for artists...")
        for artist in project_set:
            if artist["username"] in ["Lucas.Selfslagh", "Robin.Samoey", "Tom.Vandenbussche"]:
                for task_type in [t["task_type"] for t in timetrack_template["items"]]:
                    task = session.create(
                        "Task",
                        {
                            "name": f"TT_{artist['first_name']}",
                            "type": task_type,
                            "thumbnail_id": project_thumb,
                            "parent": proj,
                        },
                    )
                    session.create(
                        "Appointment",
                        {
                            "context": task,
                            "resource": artist,
                            "type": "assignment",
                        },
                    )

    drone = get_yes_no("📸 Does the project require photography?")

    session.commit()

    print(f"✅ Succesfully created {project_name} for {team['name']}")
    if deal is not None:
        print(f"Based on deal {deal}")
    print(f"--- creating objects for {project_name} on ftrack")
    print(f"--- creating directory for {project_name} on X:/")
    create_project_on_fileserver(project_name, internal_project, deal)
    print(
        f"-- {num_int} INT tasks for {int_artist['username']}"
    ) if num_int > 0 else None
    print(
        f"-- {num_ext} EXT tasks for {ext_artist['username']}"
    ) if num_ext > 0 else None
    print(
        f"-- {num_moving} MovingImage tasks for {movingImage_artist['username']}"
    ) if num_moving > 0 else None
    print("--- creating photography tasks") if drone else None
    print(f"--- creating {project_name}.mxp for 3dsmax project")
    print(f"--- updating {project_name} on teamleader")



from PyQt5 import QtWidgets
from PyQt5.QtWidgets import *

TITLE = "ftrack ams tool"
WINDOW_W = 500
WINDOW_H = 300


def user_interface(username):
    app = QtWidgets.QApplication([])
    window = QtWidgets.QMainWindow()

    window.setWindowTitle(TITLE)
    window.setGeometry(0, 0, WINDOW_W, WINDOW_H)

    # layout = QtWidgets.QVBoxLayout()
    # widget = QWidget()
    # widget.setLayout(layout)
    # window.setCentralWidget(widget)

    # label = QLabel(f"Hello, {username}! 👋")
    # layout.addWidget(label)
    # question = QLabel("What do you want to do today???? 🥳")
    # layout.addWidget(question)

    # btn_new_proj = QtWidgets.QPushButton("Create new project")
    # layout.addWidget(btn_new_proj)
    # btn_new_proj.connect()
    # new_proj = NewProjectWidget()

    # btn_add_proj = QtWidgets.QPushButton("Add to existing project")
    # layout.addWidget(btn_add_proj)

    user = UserMenu(window, username)
    new_proj = NewProjectWidget(window)
    new_proj.hide()

    user.show()
    window.show()

    user.btn_new.clicked.connect(user.hide)

    user.btn_new.clicked.connect(new_proj.show)
    app.exec_()


class UserMenu(QWidget):
    def __init__(self, parent, username):
        super(UserMenu, self).__init__(parent)
        greeting = QLabel(f"hey {username} 👋")
        question = QLabel("What are we doing today? 🤩")
        self.btn_new = QtWidgets.QPushButton("Create new project..")
        self.btn_add = QtWidgets.QPushButton("Add to existing project..")
        self.layout = QtWidgets.QGridLayout(self)
        self.layout.addWidget(greeting, 0, 0)
        self.layout.addWidget(question, 1, 0)
        self.layout.addWidget(self.btn_new, 2, 0)
        self.layout.addWidget(self.btn_add, 2, 1)
        self.setGeometry(0, 0, WINDOW_W, WINDOW_H)


class NewProjectWidget(QtWidgets.QWidget):
    def __init__(self, parent):
        super(NewProjectWidget, self).__init__(parent)
        self.layout = QtWidgets.QGridLayout(self)
        label = QLabel("Making a new project 🤩")

        projnumberlabel = QLabel("Project number:")
        self.projnumber = QLineEdit(self)

        clientcodelabel = QLabel("Client code:")
        self.clientcode = QLineEdit(self)

        projcodelabel = QLabel("Project code:")
        self.projcode = QLineEdit(self)

        self.btn_back = QPushButton("Back")
        self.btn_next = QPushButton("Next")

        self.layout.addWidget(label, 0, 0)

        self.layout.addWidget(projnumberlabel, 1, 0)
        self.layout.addWidget(self.projnumber, 1, 1)

        self.layout.addWidget(clientcodelabel, 2, 0)
        self.layout.addWidget(self.clientcode, 2, 1)

        self.layout.addWidget(projcodelabel, 3, 0)
        self.layout.addWidget(self.projcode, 3, 1)

        self.layout.addWidget(self.btn_back, 4, 0)
        self.layout.addWidget(self.btn_next, 4, 1)
        self.setGeometry(0, 0, WINDOW_W, WINDOW_H)


if __name__ == "__main__":
    user_interface("Lucas.Selfslagh")



import os
import platform
from shutil import copytree

from pick import pick

proj_dir_windows = "X:"
proj_dir_macos = "/Volumes/ams-fileserver/"

go_folder = "GO"
mv_folder = "MV"
deal_folder = "9999_Offertes_Gegevens"


def get_latest_proj():
    projectcodes = []

    dir = proj_dir_windows if platform.system() == "Windows" else proj_dir_macos

    for x in os.scandir(dir):
        if x.is_dir():
            try:
                code = int(x.name[0:4])
            except Exception:
                continue
            else:
                if code != 9999:
                    projectcodes.append(code)

    return max(projectcodes) + 1


def get_deal_info(dealnumber):
    dir = proj_dir_windows if platform.system() == "Windows" else proj_dir_macos
    deal_dir = os.path.join(dir, deal_folder)
    deal = None
    deals = [f for f in os.listdir(deal_dir) if f"_{dealnumber}_" in f]

    if len(deals) > 1:
        ret = pick(deals, "pick a deal")
        deal = deals[ret[1]]
    elif len(deals) == 1:
        deal = deals[0]
    else:
        print(f'no deal found in {deal_folder}')

    if deal is not None:
        return deal


def create_project_on_fileserver(project, internal=False, deal=None):

    dir = proj_dir_windows if platform.system() == "Windows" else proj_dir_macos
    project_dir = f"{dir}/{project}"

    print(project_dir)
    print(f"{project_dir} already exists") if os.path.isdir(
        project_dir
    ) else os.makedirs(project_dir)
    project_go = os.path.join(project_dir, go_folder)
    print(f"{project_go} already exists") if os.path.isdir(
        project_go) else os.makedirs(project_go)
    project_mv = os.path.join(project_dir, mv_folder)
    print(f"{project_mv} already exists") if os.path.isdir(
        project_mv) else os.makedirs(project_mv)

    if deal is not None:
        deal_dir = os.path.join(dir, deal_folder, deal)
        deal_go = os.path.join(deal_dir, "GO")
        deal_mv = os.path.join(deal_dir, "MV")

        if os.path.isdir(deal_go):
            copytree(deal_go, project_go, dirs_exist_ok=True)

        if os.path.isdir(deal_mv):
            copytree(deal_mv, project_mv, dirs_exist_ok=True)
        else:
            print("probber on MV proj")

    if internal:
        maps = os.path.join(project_dir, "Maps")
        print(f"{maps} already exists") if os.path.isdir(maps) else os.makedirs(maps)
        scenes = os.path.join(project_dir, "Scenes")
        print(f"{scenes} already exists") if os.path.isdir(
            scenes) else os.makedirs(scenes)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ftrack_ams']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5>=5.15.9,<6.0.0',
 'click>=8.0.3,<9.0.0',
 'ftrack-python-api>=2.3.1,<3.0.0',
 'pick>=1.2.0,<2.0.0']

entry_points = \
{'console_scripts': ['ftrack-ams = ftrack_ams.console:main']}

setup_kwargs = {
    'name': 'ftrack-ams',
    'version': '2.0.27',
    'description': '',
    'long_description': 'None',
    'author': 'Lucas Selfslagh',
    'author_email': 'lucas@animotions.be',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<=3.11.4',
}


setup(**setup_kwargs)
